﻿using Hotline.Push.FWMessage;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Push;
using Hotline.Share.Dtos.Push.FWMessage;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Filters;
using XF.Domain.Repository;

namespace Hotline.Api.Controllers
{
    public class PushMessageController : BaseController
    {
        #region 注入
        private readonly IRepository<Message> _messageRepository;
        private readonly IMapper _mapper;
        private readonly IPushDomainService _pushDomainService;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="messageRepository"></param>
        /// <param name="mapper"></param>
        /// <param name="pushDomainService"></param>
        public PushMessageController(IRepository<Message> messageRepository, IMapper mapper, IPushDomainService pushDomainService)
        {
            _messageRepository = messageRepository;
            _mapper = mapper;
            _pushDomainService = pushDomainService;
        } 
        #endregion

        #region 新增
        /// <summary>
        /// 新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPut("add")]
        public async Task AddMessage([FromBody] MessageDto dto)
        {
            await _pushDomainService.PushAsync(dto, HttpContext.RequestAborted);
        }
        #endregion

        #region 查询短信剩余数量
        /// <summary>
        /// 查询短信剩余数量
        /// </summary>
        /// <returns></returns>
        [HttpGet("query")]
        public async Task<string> GetAccountNum()
        {
            return await _pushDomainService.GetAccountNum();
        }
        #endregion

        #region 短信发送状态回调接口
        /// <summary>
        /// 短信发送状态回调接口
        /// </summary>
        /// <param name="receiveMessageDto"></param>
        /// <returns></returns>
        [HttpPost("receiveobtain")]
        [AllowAnonymous]
        public async Task<OpenResponse> ReceiveObtain([FromForm] ReceiveMessageDto receiveMessageDto)
        {
            var result = await _pushDomainService.ReceiveObtain(receiveMessageDto);

            return OpenResponse.Ok(result);
        }
        #endregion

        #region 短信接收
        /// <summary>
        /// 短信接收
        /// </summary>
        /// <param name="receiveMessageDto"></param>
        /// <returns></returns>
        [HttpPost("receivesms")]
        [AllowAnonymous]
        public async Task<OpenResponse> ReceiveSms([FromForm] ReceiveMessageDto receiveMessageDto)
        {
            var result = await _pushDomainService.ReceiveSms(receiveMessageDto);

            return OpenResponse.Ok(result);
        }
        #endregion

        #region 查询短信
        /// <summary>
        /// 查询短信
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpGet("getlist")]
        public async Task<PagedDto<MessageDataDto>> AllMessage([FromQuery] MessagePagedDto pagedDto)
        {
            var (total, items) = await _messageRepository
                    .Queryable()
                    .Includes(it => it.User)
                    .Includes(it => it.SystemOrganize)
                    .Includes(it => it.Order)
                    .WhereIF(pagedDto.PushBusiness.HasValue, d => d.PushBusiness == pagedDto.PushBusiness)
                    .WhereIF(pagedDto.Status.HasValue, d => d.Status == pagedDto.Status)
                    .WhereIF(!string.IsNullOrEmpty(pagedDto.Keyword), d => d.Name.Contains(pagedDto.Keyword!) || d.Content.Contains(pagedDto.Keyword!) || d.TelNumber.Contains(pagedDto.Keyword!) || d.User.Name.Contains(pagedDto.Keyword!) || d.SystemOrganize.Name.Contains(pagedDto.Keyword!))
                    .WhereIF(pagedDto.StartTime.HasValue, d => d.SendTime >= pagedDto.StartTime)
                    .WhereIF(pagedDto.EndTime.HasValue, d => d.SendTime <= pagedDto.EndTime)
                    .OrderByDescending(it => it.CreationTime)
                    .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize, HttpContext.RequestAborted);

            return new PagedDto<MessageDataDto>(total, _mapper.Map<IReadOnlyList<MessageDataDto>>(items));

        }

        #endregion

    }
}
