﻿using Hotline.FlowEngine.Definitions;
using Hotline.Orders;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.FlowEngine.Definition;
using Hotline.Share.Dtos.Settings;

namespace Hotline.Application.FlowEngine
{
    public interface IWorkflowApplication
    {
        /// <summary>
        /// 开始流程
        /// </summary>
        Task<string> StartWorkflowAsync(StartWorkflowDto dto, string externalId, TimeResult? expiredTime = null,
            CancellationToken cancellationToken = default);

        /// <summary>
        /// 流转至下一节点（节点办理）
        /// </summary>
        Task NextAsync(NextWorkflowDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 撤回至任意节点
        /// </summary>
        Task RecallAsync(RecallDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 撤回至派单节点
        /// </summary>
        Task RecallToSendAsync(string workflowId, string opinion, CancellationToken cancellationToken);

        /// <summary>
        /// 跳转至任意节点
        /// </summary>
        Task JumpAsync(RecallDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 重办
        /// </summary>
        Task RedoAsync(RecallDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 查询流程开始节点的下一节点配置
        /// </summary>
        Task<DefinedStepDto> GetStartOptionsAsync(string moduleCode, CancellationToken cancellationToken);

        /// <summary>
        /// 查询节点配置可选参数
        /// </summary>
        Task<IReadOnlyList<Kv>> GetNextStepOptionsAsync(StepDefineBasic stepDefine, CancellationToken cancellationToken);

        ////////

        /// <summary>
        /// 查询开始流程的下一步待选节点
        /// </summary>
        Task<NextStepsDto> GetStartStepsAsync(string moduleCode, CancellationToken cancellationToken);

        /// <summary>
        /// 查询办理流程的下一步待选节点
        /// </summary>
        Task<NextStepsDto> GetNextStepsAsync(string workflowId, CancellationToken cancellationToken);

        /// <summary>
        /// 查询撤回可选节点
        /// </summary>
        Task<NextStepsDto> GetRecallStepsAsync(string workflowId, CancellationToken cancellationToken);

        /// <summary>
        /// 查询跳转可选节点
        /// </summary>
        Task<NextStepsDto> GetJumpStepsAsync(string workflowId, CancellationToken cancellationToken);

        /// <summary>
        /// 查询重办可选节点
        /// </summary>
        Task<NextStepsDto> GetRedoStepsAsync(string workflowId, CancellationToken cancellationToken);

        /// <summary>
        /// 否决
        /// </summary>
        Task RejectAsync(RejectDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 查询下一步所选节点对应待选项
        /// </summary>
        Task<IReadOnlyList<Kv>> GetNextStepOptionsAsync(GetNextStepItemsDto dto, CancellationToken cancellationToken);
    }
}