﻿using Hotline.FlowEngine.Workflows;
using Hotline.Orders;
using Hotline.Settings.TimeLimits;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using Hotline.Users;
using XF.Domain.Dependency;
using XF.Domain.Entities;
using XF.Domain.Exceptions;

namespace Hotline.Application.Orders;

public class OrderApplication : IOrderApplication, IScopeDependency
{
    private readonly IWorkflowDomainService _workflowDomainService;
    private readonly IOrderRepository _orderRepository;
    private readonly ITimeLimitDomainService _timeLimitDomainService;

    public OrderApplication(
        IWorkflowDomainService workflowDomainService,
        IOrderRepository orderRepository,
        ITimeLimitDomainService timeLimitDomainService
        )
    {
        _workflowDomainService = workflowDomainService;
        _orderRepository = orderRepository;
        _timeLimitDomainService = timeLimitDomainService;
    }

    /// <summary>
    /// 更新工单办理期满时间
    /// 1.更新工单 2.更新流程 3.推送省平台
    /// </summary>
    /// <returns></returns>
    public async Task DelayOrderHandleExpiredTimeAsync(Order order, ETimeType timeType, int timeCount, bool publishToProvince = true)
    {
        var expiredTime = _timeLimitDomainService.CalcEndTime(order.ExpiredTime.Value, timeType, timeCount, false);
        order.ExpiredTime = expiredTime.EndTime;


    }
}