﻿using Hotline.Quality;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Share.Dtos.File;
using Hotline.Share.Dtos.Quality;
using Hotline.Share.Enums.Quality;
using MapsterMapper;
using Microsoft.AspNetCore.Http;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Repository.SqlSugar.Quality
{
	public class QualityRepository : BaseRepository<Hotline.Quality.Quality>, IQualityRepository, IScopeDependency
	{
		private readonly ISessionContext _sessionContext;
		private readonly IMapper _mapper;
		private readonly IRepository<QualityDetail> _qualiteyDetail;

		public QualityRepository(
			ISugarUnitOfWork<HotlineDbContext> uow,
			IDataPermissionFilterBuilder dataPermissionFilterBuilder,
			ISessionContext sessionContext,
			IMapper mapper,
			IRepository<QualityDetail> qualiteyDetail
			) : base(uow, dataPermissionFilterBuilder)
		{
			_sessionContext = sessionContext;
			_mapper = mapper;
			_qualiteyDetail = qualiteyDetail;
		}

		public async Task AddQualityAsync(EQualitySource Source, string OrderId, string VisitId , CancellationToken cancellationToken)
		{
			QualityAddDto quality = new QualityAddDto
			{
				Source = Source,
				OrderId = OrderId,
				VisitId = VisitId	
			};
			var count = await CountAsync(x => x.OrderId == OrderId && x.Source == Source, cancellationToken);
			if (count <= 0)
				await AddQualityAsync(quality, cancellationToken);
		}

		public async Task AddQualityAsync(EQualitySource Source, string OrderId, CancellationToken cancellationToken) 
		{
			QualityAddDto quality =  new  QualityAddDto
			{
				Source = Source,
				OrderId = OrderId
			};
			var count = await CountAsync(x => x.OrderId == OrderId && x.Source == Source, cancellationToken);
			if (count <= 0)
				await AddQualityAsync(quality, cancellationToken);
		}

		public async Task AddQualityAsync(QualityAddDto model, CancellationToken cancellationToken)
		{
			var quality = _mapper.Map<Hotline.Quality.Quality>(model);
			var id = await AddAsync(quality, cancellationToken);
			if (model.QualityDetails.Any())
			{
				foreach (var item in model.QualityDetails)
				{
					item.QualityId = id;
				}
				List<QualityDetail> details = _mapper.Map<List<QualityDetail>>(model.QualityDetails);
				await _qualiteyDetail.AddRangeAsync(details, cancellationToken);
			}
		}

		public async Task UpdateQualityAsync(UpdateQualityDto model, CancellationToken cancellationToken)
		{
			//验证工单是否可以申请
			var quality = await GetAsync(model.Id, cancellationToken);
			if (quality is null)
				throw UserFriendlyException.SameMessage("无效质检");
			_mapper.Map(model, quality);
			quality.State = EQualityState.End;
			quality.UserName = _sessionContext.UserName;
			quality.QualityTime = DateTime.Now;
			await UpdateAsync(quality, cancellationToken);
			List<QualityDetail> deleteDetails = await _qualiteyDetail.Queryable().Where(x => x.QualityId == model.Id).ToListAsync(cancellationToken);
			await _qualiteyDetail.RemoveRangeAsync(deleteDetails, cancellationToken);
			if (model.QualityDetails.Any())
			{
				foreach (var item in model.QualityDetails)
				{
					item.QualityId = model.Id;
				}
				List<QualityDetail> details = _mapper.Map<List<QualityDetail>>(model.QualityDetails);
				await _qualiteyDetail.AddRangeAsync(details, cancellationToken);
			}
		}
	}
}
