﻿using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.Org;
using Hotline.Share.Dtos.Quality;
using Hotline.Share.Enums.KnowledgeBase;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Knowledge
{
    /// <summary>
    /// 编辑
    /// </summary>
    public record UpdateKnowledgeDto : AddKnowledgeDto
    {
        /// <summary>
        /// ID
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 标签名称
        /// </summary>
        public List<string> Tags { get; set; } = new();
	}

    /// <summary>
    /// 新增知识
    /// </summary>
    public record AddKnowledgeDto : KnowledgeDto
    {
        /// <summary>
        /// 知识Code
        /// </summary>
        public string Code { get; set; }

        /// <summary>
        /// 标签名称
        /// </summary>
        public List<string> Tags { get; set; } = new();

        /// <summary>
        /// 知识归属
        /// </summary>
        public string? Attribution { get; set; }

        /// <summary>
        /// 来源部门
        /// </summary>
        public string? SourceOrganizeId { get; set; }

	}

    /// <summary>
    /// 查询详情
    /// </summary>
    public record KnowledgeInfoDto : KnowledgeDto
    {
        /// <summary>
        /// ID
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 知识Code
        /// </summary>
        public string Code { get; set; }

        /// <summary>
        /// 知识分类名称
        /// </summary>
        public string KnowledgeTypeName { get; set; }

        /// <summary>
        /// 热点名称
        /// </summary>
        public string HotspotName { get; set; }

        /// <summary>
        /// 流程Id
        /// </summary>
        public string? WorkflowId { get; set; }

        /// <summary>
        /// 是否可办理
        /// </summary>
        public bool? IsCanHandle { get; set; }
	}

    /// <summary>
    /// 基础
    /// </summary>
    public record KnowledgeDto : KnowledgeBaseDto
	{
        /// <summary>
        /// 知识分类
        /// </summary>
        public string KnowledgeTypeId { get; set; }

        public string KnowledgeTypeText => KnowledgeType != null ? KnowledgeType.SpliceName : "-";

		/// <summary>
		/// 热点
		/// </summary>
		public string HotspotId { get; set; }

        /// <summary>
        /// 标题
        /// </summary>
        public string Title { get; set; }

        /// <summary>
        /// 摘要
        /// </summary>
        public string Summary { get; set; }

        /// <summary>
        /// 内容
        /// </summary>
        public string Content { get; set; }

        /// <summary>
        /// 是否公开
        /// </summary>
        public bool IsPublic { get; set; }

        /// <summary>
        /// 文档状态,默认草稿
        /// </summary>
        public EKnowledgeStatus Status { get; set; } = EKnowledgeStatus.Drafts;

        public string StatusText => Status.GetDescription();

		/// <summary>
		/// 到期时间
		/// </summary>
		public DateTime? ExpiredTime { get; set; }

        /// <summary>
        /// 关键词
        /// </summary>
        public List<string> Keywords { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public List<string> Additions { get; set; }

        /// <summary>
        /// 外部数据（为前端提供级联功能）
        /// </summary>
        public string HotspotExternal { get; set; }

        /// <summary>
        /// 知识归属
        /// </summary>
        public string? Attribution { get; set; }

        /// <summary>
        /// 浏览量
        /// </summary>
        public int PageView { get; set; } = 0;

		/// <summary>
		/// 搜索量
		/// </summary>
		public int? SearchNum { get; set; } = 0;

		/// <summary>
		/// 评分
		/// </summary>
		public decimal? Score { get; set; } = decimal.Zero;

        /// <summary>
        /// 评论数
        /// </summary>
        public int? CommentNum { get; set; } = 0;

        /// <summary>
        /// 来源部门
        /// </summary>
        public string? SourceOrganizeId { get; set; }

		/// <summary>
		/// 来源部门
		/// </summary>
		public OrgDto SourceOrganize { get; set; }

        public KnowledgeTypeDto KnowledgeType { get; set; }

        public KnowledgeCollectDto Collect { get; set; }

        /// <summary>
        /// 关联知识
        /// </summary>
        public List<string> Knowledges { get; set; }
		/// <summary>
		/// 关联知识对象
		/// </summary>
		public List<KnowledgeDto> KnowledgeDtos { get; set; }

		/// <summary>
		/// 关键词
		/// </summary>
		public List<KnowledgeWordDto> KeywordsDto { get; set; }

	}

    public record KnowledgeBaseDto {
		public DateTime? LastModificationTime { get; set; }

		public bool IsDeleted { get; set; }

		/// <summary>
		/// 删除时间
		/// </summary>
		public DateTime? DeletionTime { get; set; }


		/// <summary>
		/// 创建时间
		/// </summary>
		public DateTime CreationTime { get; set; }

		public string Id { get; set; }

		/// <summary>
		/// 组织Id
		/// </summary>
		public string? CreatorOrgId { get; set; }


		public string? CreatorOrgName { get; set; }

		/// <summary>
		/// 创建人
		/// </summary>
		public string? CreatorId { get; set; }

		public string? CreatorName { get; set; }
	}

    public class KnowledgeTitleDto {

		/// <summary>
		/// ID
		/// </summary>
		public string? Id { get; set; }

		/// <summary>
		/// 标题
		/// </summary>
		public string Title { get; set; }

	}

    public class KnowledgeSearchNumDto
	{

	    /// <summary>
	    /// ID
	    /// </summary>
	    public string? Id { get; set; }

    }


	public class AddStartFlowDto : StartWorkflowDto<AddKnowledgeDto>
    {

    }

    public class UpdateStartFlowDto : StartWorkflowDto<UpdateKnowledgeDto>
    {

    }

    public class DeleteStartFlowDto : StartWorkflowDto<KnowledgeDto>
    {

    }
}
