﻿using Hotline.Share.Enums.Order;
using Hotline.Share.Requests;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Order;

public class PublishedDto
{
    public string Id { get; set; }

    public OrderDto Order { get; set; }

    /// <summary>
    /// 发布范围
    /// </summary>
    public bool? PublishState { get; set; }

    /// <summary>
    /// 整理内容
    /// </summary>
    public string? ArrangeContent { get; set; }

    /// <summary>
    /// 发布时间
    /// </summary>
    public DateTime? PublishTime { get; set; }

    public string? PublishMan { get; set; }
}

public class PublishDto : OrderDto
{
    /// <summary>
    /// 发布范围
    /// </summary>
    public bool? PublishState { get; set; }

    /// <summary>
    /// 整理内容
    /// </summary>
    public string? ArrangeContent { get; set; }

    /// <summary>
    /// 发布时间
    /// </summary>
    public DateTime? PublishTime { get; set; }

    public string? PublishMan { get; set; }
}


public class PublishOrderPageBaseDto
{
    /// <summary>
    /// 来源 0：热线平台  1：省平台 2：110
    /// </summary>
    public string SourceChannel { get; set; }

    /// <summary>
    /// 工单标题
    /// </summary>
    public string OrderTitle { get; set; }

    /// <summary>
    /// 工单内容
    /// </summary>
    public string Content { get; set; }

    /// <summary>
    /// 办理结果
    /// </summary>
    public string ActualOpinion { get; set; }

    /// <summary>
    /// 实际办理部门名称
    /// </summary>
    public Kv ActualHandleOrgName { get; set; }

    /// <summary>
    /// 需回访部门
    /// </summary>
    public List<Kv>? idNames { get; set; }
}


public class PublishOrderDto
{
    /// <summary>
    /// 工单ID
    /// </summary>
    public string Id { get; set; }

    /// <summary>
    /// 发布范围
    /// </summary>
    public bool PublishState { get; set; }

    /// <summary>
    /// 整理标题
    /// </summary>
    public string ArrangeTitle { get; set; }

    /// <summary>
    /// 整理内容
    /// </summary>
    public string ArrangeContent { get; set; }

    /// <summary>
    /// 整理结果
    /// </summary>
    public string ArrangeOpinion { get; set; }

    /// <summary>
    /// 需回访部门
    /// </summary>
    public List<Kv> IdNames { get; set; }

    #region 省工单使用

    /// <summary>
    /// 省是否公开
    /// </summary>
    public bool? ProPublishState { get; set; }
    /// <summary>
    /// 反馈人电话
    /// </summary>
    public string? FeedBackPhone { get; set; }

    /// <summary>
    /// 不公开原因
    /// </summary>
    public string? NoPubReason { get; set; }

    #endregion
}


public class PublishOrderModifyDto
{
    /// <summary>
    /// 发布ID
    /// </summary>
    public string Id { get; set; }


    /// <summary>
    /// 整理标题
    /// </summary>
    public string ArrangeTitle { get; set; }

    /// <summary>
    /// 整理内容
    /// </summary>
    public string ArrangeContent { get; set; }

    /// <summary>
    /// 整理结果
    /// </summary>
    public string ArrangeOpinion { get; set; }
}


public class OrderVisitDto
{
    public string Id { get; set; }

    public OrderDto Order { get; set; }

    /// <summary>
    /// 回访状态
    /// </summary>
    public EVisitState VisitState { get; set; }

    public string VisitStateText => VisitState.GetDescription();

    /// <summary>
    /// 回访方式
    /// </summary>
    public EVisitType? VisitType { get; set; }

    public string VisitTypeText => VisitType?.GetDescription() ?? string.Empty;


    /// <summary>
    /// 发布时间
    /// </summary>
    public DateTime PublishTime { get; set; }

    /// <summary>
    /// 回访人
    /// </summary>
    public string? EmployeeId { get; set; }

    public string? EmployeeName { get; set; }

    public string CreationTime { get; set; }

    /// <summary>
    /// 是否接通
    /// </summary>
    public bool? IsPutThrough { get; set; }

    /// <summary>
    /// 重办状态
    /// </summary>
    public EAgainState? AgainState { get; set; }

    public string AgainStateText => AgainState?.GetDescription() ?? string.Empty;

    /// <summary>
    /// 回访时间
    /// </summary>
    public DateTime? VisitTime { get; set; }

    /// <summary>
    /// 回访明细
    /// </summary>
    public List<OrderVisitDetailDto> OrderVisitDetails { get; set; }
}

public class OrderVisitDetailDto
{

    /// <summary>
    /// 回访主表明细ID
    /// </summary>
    public string Id { get; set; }

    /// <summary>
    /// 回访主表ID
    /// </summary>
    public string VisitId { get; set; }


    public OrderVisitDto OrderVisit { get; set; }

    public OrderDto Order => OrderVisit != null ? OrderVisit.Order : null;

    /// <summary>
    /// 语音评价（话务评价）
    /// </summary>
    public EVoiceEvaluate? VoiceEvaluate { get; set; }

    public string? VoiceEvaluateText => VoiceEvaluate?.GetDescription();


    /// <summary>
    /// 话务员评价（话务评价）
    /// </summary>
    public ESeatEvaluate? SeatEvaluate { get; set; }

    public string? SeatEvaluateText => SeatEvaluate?.GetDescription();

    /// <summary>
    /// 部门办件结果
    /// </summary>
    public Kv? OrgProcessingResults { get; set; }

    /// <summary>
    /// 不满意原因
    /// </summary>
    public List<Kv>? OrgNoSatisfiedReason { get; set; }

    /// <summary>
    /// 部门办件态度
    /// </summary>
    public Kv? OrgHandledAttitude { get; set; }

    /// <summary>
    /// 回访内容
    /// </summary>
    public string? VisitContent { get; set; }

    /// <summary>
    /// 回访部门Code
    /// </summary>
    public string VisitOrgCode { get; set; }

    /// <summary>
    /// 回访部门名称
    /// </summary>
    public string VisitOrgName { get; set; }

    /// <summary>
    /// 回访对象类型 10：话务员 20：部门
    /// </summary>
    public EVisitTarget VisitTarget { get; set; }
}

public class DistributionVisitRspDto
{
    public int SuccessCount { get; set; }

    public int ErrorCount { get; set; }
}

public class PublishVisitDto
{
    /// <summary>
    /// 工单对象
    /// </summary>
    public OrderDto Order { get; set; }

    /// <summary>
    /// 服务工单编号
    /// </summary>
    public string No { get; set; }

    /// <summary>
    /// 回访评价方式
    /// </summary>
    public EVisitType? VisitType { get; set; }

    /// <summary>
    /// 回访人
    /// </summary>
    public string VisitName { get; set; }

    /// <summary>
    /// 回访时间
    /// </summary>
    public DateTime? VisitTime { get; set; }

    /// <summary>
    /// 回访评价情况
    /// </summary>
    public string VisitRemark { get; set; }

    /// <summary>
    /// 回访结果满意度
    /// </summary>
    public string SubjectResultSatifyCode { get; set; }

    /// <summary>
    /// 行政区划代码
    /// </summary>
    public string AreaCode { get; set; }

    /// <summary>
    /// 工单回访首次结果满意度
    /// </summary>
    public string FirstSatisfactionCode { get; set; }

    /// <summary>
    /// 附件
    /// </summary>
    public string ClientGuid { get; set; }
}

public class SuperviseOrderDto
{

    /// <summary>
    /// 督办ID
    /// </summary>
    public string Id { get; set; }

    /// <summary>
    /// 工单ID
    /// </summary>
    public string OrderId { get; set; }

    /// <summary>
    /// 督办回复时限
    /// </summary>
    public DateTime ReplyLimitTime { get; set; }

    /// <summary>
    /// 被督办部门ID
    /// </summary>
    public string OrgId { get; set; }

    /// <summary>
    /// 被督办部门名称
    /// </summary>
    public string OrgName { get; set; }


    /// <summary>
    /// 创建部门名称
    /// </summary>
    public string? CreatorOrgName { get; set; }

    /// <summary>
    /// 督办状态
    /// </summary>
    public int State { get; set; }

    public string StateText => State == 0 ? "待办未读" : State == 1 ? "已回复" : State == 2 ? "签收已读" : "-";

	/// <summary>
	/// 督办签收时间
	/// </summary>
	public DateTime? SignTime { get; set; }

    /// <summary>
    /// 督办回复时间
    /// </summary>
    public DateTime? ReplyTime { get; set; }

    /// <summary>
    /// 督办申请内容
    /// </summary>
    public string? ApplyContent { get; set; }

    /// <summary>
    /// 督办回复内容
    /// </summary>
    public string? ReplyContent { get; set; }

    /// <summary>
    /// 回复人
    /// </summary>
    public string? ReplyId { get; set; }

    /// <summary>
    /// 附件
    /// </summary>
    public List<string>? Additions { get; set; } = new();

    public DateTime? LastModificationTime { get; set; }

    public string? CreatorId { get; set; }

    public string? CreatorOrgId { get; set; }

    public DateTime CreationTime { get; set; }

    /// <summary>
    /// 工单对象
    /// </summary>
    public OrderDto Order { get; set; }
    public string ReUser { get; set; }
    public string CrUser { get; set; }
    public string SignUser { get; set; }

    /// <summary>
    /// 督办省编号
    /// </summary>
    public string? SuperviseSerial { get; set; }
}


public class UrgeOrderDto
{

    /// <summary>
    /// 催办ID
    /// </summary>
    public string Id { get; set; }

    /// <summary>
    /// 工单ID
    /// </summary>
    public string OrderId { get; set; }

    /// <summary>
    /// 催办回复时限
    /// </summary>
    public DateTime ReplyLimitTime { get; set; }

    /// <summary>
    /// 被催办部门ID
    /// </summary>
    public string OrgId { get; set; }

    /// <summary>
    /// 被催办部门名称
    /// </summary>
    public string OrgName { get; set; }

    /// <summary>
    /// 创建部门名称
    /// </summary>
    public string? CreatorOrgName { get; set; }

    /// <summary>
    /// 催办状态
    /// </summary>
    public int State { get; set; }

    public string StateText => State == 0 ? "待办未读" : State == 1 ? "已回复" : State == 2 ? "签收已读" : "-";

	/// <summary>
	/// 催办签收时间
	/// </summary>
	public DateTime? SignTime { get; set; }

    /// <summary>
    /// 催办回复时间
    /// </summary>
    public DateTime? ReplyTime { get; set; }

    /// <summary>
    /// 催办申请内容
    /// </summary>
    public string? ApplyContent { get; set; }

    /// <summary>
    /// 催办回复内容
    /// </summary>
    public string? ReplyContent { get; set; }

    /// <summary>
    /// 回复人
    /// </summary>
    public string? ReplyId { get; set; }

    /// <summary>
    /// 附件
    /// </summary>
    public List<string>? Additions { get; set; } = new();

    public DateTime? LastModificationTime { get; set; }

    public string? CreatorId { get; set; }

    public string? CreatorOrgId { get; set; }

    public DateTime CreationTime { get; set; }

    /// <summary>
    /// 工单对象
    /// </summary>
    public OrderDto Order { get; set; }

    public string ReUser { get; set; }

    public string CrUser { get; set; }

    public string SignUser { get; set; }
}

public class AboutToExpireDto : OrderDto
{
    public string IntervalText { get; set; }

}

public record AboutToExpireListDto : PagedKeywordRequest
{
    /// <summary>
    /// 是否为省工单
    /// </summary>
    public bool? IsProvince { get; set; }

    /// <summary>
    /// 工单编码（20220101000001）
    /// </summary>
    public string? No { get; set; }

    /// <summary>
    /// 工单标题
    /// </summary>
    public string? Title { get; set; }
}