﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.FlowEngine.Workflows;
using Hotline.Share.Enums.Settings;
using SqlSugar;
using XF.Domain.Repository;

namespace Hotline.Orders
{
    /// <summary>
    /// 特提
    /// </summary>
    public class OrderSpecial : FullStateEntity
	{
        public string OrderId { get; set; }

        /// <summary>
        /// 节点Id
        /// </summary>
        public string TraceId { get; set; }

        /// <summary>
        /// 特提原因
        /// </summary>
        [SugarColumn(ColumnDescription = "特提原因")]
        public string? Cause { get; set; }

		/// <summary>
		/// 特提理由
		/// </summary>
		[SugarColumn(ColumnDescription = "特提理由", ColumnDataType = "varchar(2000)")]
        public string? Reason { get; set; }

        /// <summary>
        /// 办理时间限制(如：24小时、7个工作日)
        /// </summary>
        public int? TimeLimit { get; set; }

        public ETimeType? TimeLimitUnit { get; set; } = ETimeType.WorkDay;

        /// <summary>
        /// 是否修改期满时间
        /// </summary>
        public bool AlterTime { get; set; }

		/// <summary>
		/// 特提意见
		/// </summary>
		[SugarColumn(ColumnDescription = "特提意见", ColumnDataType = "varchar(2000)")]
        public string? Opinion { get; set; }

		/// <summary>
		///  审核结果  0  待审核  1 审核通过  2 审核不通过
		/// </summary>
		public int State { get; set; }

		[SugarColumn(ColumnDescription = "组织Id")]
		public string? OrgId { get; set; }

		public string? OrgName { get; set; }

		/// <summary>
		/// 特提节点
		/// </summary>
		[Navigate(NavigateType.OneToOne, nameof(TraceId))]
		public WorkflowTrace Trace { get; set; }

        /// <summary>
        /// 工单
        /// </summary>
        [Navigate(NavigateType.OneToOne, nameof(OrderId))]
        public Order Order { get; set; }
	}
}
